/*
    Copyright 2008, 2009, 2010 Denis Comtesse <comtesse@users.sourceforge.net>

    This file is part of F.L.A.W.

    F.L.A.W. is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    F.L.A.W. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with F.L.A.W.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "misc.h"

using namespace std;

Uint32 GetPixel(SDL_Surface *surface, int x, int y)
{
    int bpp = surface->format->BytesPerPixel;
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;
	
    switch(bpp) {
    case 1:
        return *p;
        break;

    case 2:
        return *(Uint16 *)p;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;
        break;

    case 4:
        return *(Uint32 *)p;
        break;

    default:
        return 0;      
    }
}

//Replacement for SDL_Delay which causes problems with screensavers
void CustomWait(Uint32 ms) {
	SDL_Event event;
    Uint32 startticks, ticks;
    startticks = SDL_GetTicks();
    do {
        SDL_PollEvent(&event); //do something
        ticks = SDL_GetTicks();
    } while (ticks - startticks < ms);
}


int randomnumber(int a, int b) {
	double r = b - a + 1;
	return a + (int)(r * rand()/(RAND_MAX+1.0));
}

Sint8 Sgn(float a){
	Sint8 s;
	if (a < 0){
		s = -1;
	}
	else if (a > 0){
		s = 1;
	}
	else{
		s = 0;
	}
	return s;
}

bool Fonts::LoadFonts(const char *path, float fac) {
	char filename[100];
	sprintf (filename, "%sFreeSans.ttf", path);
	if (!(smallfont = TTF_OpenFont(filename, 12*fac))) return false;
	if (!(tinyfont = TTF_OpenFont(filename, 9))) return false;
	if (!(normalfont = TTF_OpenFont(filename, 24*fac))) return false;
	sprintf (filename, "%sFreeSansBold.ttf", path);
	if (!(boldfont = TTF_OpenFont(filename, 24*fac))) return false;
	return true;
}


//EXPERIMENTAL: 

//reduce alpha blending
SDL_Surface *killalpha (SDL_Surface *surface, Uint8 alpha){ // remove alpha channel, use color key instead
	SDL_Surface *dummy;
	dummy = SDL_CreateRGBSurface(SDL_SRCCOLORKEY, 1, 1, 32, 0xff000000, 0x00ff0000, 0x0000ff00, 0x00000000);
	surface = SDL_ConvertSurface(surface, dummy->format, SDL_SRCCOLORKEY);
	Uint32 pixel = GetPixel(surface, 0, 0);
	Uint8 R,G,B;
	SDL_GetRGB(pixel, surface->format, &R, &G, &B);
	SDL_SetColorKey(surface, SDL_SRCCOLORKEY, (R,G,B));
	SDL_SetAlpha(surface, SDL_SRCALPHA|SDL_RLEACCEL, alpha);
	return surface;
}


/* // *** Render Text with SDL_Pango ***
 
SDL_Surface *RenderText (const char *text, Uint8 size, const char *color, bool bold){
	if (SDLPango_WasInit() == 0) SDLPango_Init();
	SDLPango_Context *context = SDLPango_CreateContext();
	SDLPango_SetDefaultColor(context, MATRIX_TRANSPARENT_BACK_WHITE_LETTER);
	char markup[400];
	if (!bold) sprintf (markup, "<span font='%d' color='%s'>%s</span>", size, color, text);
	else sprintf (markup, "<span font='%d' color='%s'><b>%s</b></span>", size, color, text);
	SDLPango_SetMarkup(context, markup, -1);
	Uint32 w = SDLPango_GetLayoutWidth(context);
    Uint32 h = SDLPango_GetLayoutHeight(context);
    Uint8 mx = 0; //margin
    Uint8 my = 0;
    SDL_Surface *surface;
	surface = SDL_CreateRGBSurface(SDL_SWSURFACE, w+mx*2, h+my*2, 32, 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
	SDLPango_Draw(context, surface, mx, my);
	SDLPango_FreeContext(context);
	return surface;
}
*/


